package net.metadiversity.diversity.navigator.connection;

import java.util.*;
import javax.xml.parsers.*;

import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;

import java.io.File;

import net.metadiversity.diversity.navigator.io.*;
import net.metadiversity.diversity.navigator.util.Crypt;

import org.apache.log4j.Logger;

/*  all informations about connections are stored in an xml-file "connection.xml"
 *  located in user-home directory
 *
 */
public class ConnectionsManager 
{
    static Logger logger = Logger.getLogger( ConnectionsManager.class.getName() );
    static String configFileName;
    
    private static LinkedHashMap connections;
    
    final static Crypt crypt = new Crypt();
    
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public ConnectionsManager()
    {
        // read file from user.home directory
	String path = System.getProperty( "user.home" );
        configFileName = path + "/.diversity_navigator/diversity_navigator.connections.xml";
	connections = new LinkedHashMap();                
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public static ConnectionInfo getConnectionInfoByName( String connectionName )
    {
        ConnectionInfo ci = null;
        try
        {
            if( connections == null )
            {
                getConnections();
            }
            ci = ( ConnectionInfo ) connections.get( connectionName );
        }
        catch( java.lang.Exception ex )        
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );     
        }
        return ci;
    }
    public static LinkedHashMap getConnections() throws Exception 
    {
        // read file from user.home directory
        if( configFileName == null )
        {
            String path = System.getProperty( "user.home" );
            configFileName = path + "/.diversity_navigator/diversity_navigator.connections.xml";
        }    
        // test if connection-file exist        
	File f = new java.io.File( configFileName );
	if( ! f.exists() ) 
        {
             logger.error( "Can't read file: " + configFileName ); 
             return new LinkedHashMap();
        }
	connections = new LinkedHashMap();                
        // get XML factory
        try
        {
            javax.xml.parsers.DocumentBuilderFactory factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
            factory.setValidating( true );        // enable Validating with DTD
            javax.xml.parsers.DocumentBuilder docBuilder = factory.newDocumentBuilder();
            docBuilder.setErrorHandler( new XmlSaxErrorHandler() ); 
            org.w3c.dom.Document doc = docBuilder.parse( configFileName );
            Element root = doc.getDocumentElement();
            NodeList nodeList = root.getElementsByTagName( "connection" );
            for( int i = 0; i < nodeList.getLength(); i++ ) 
            {
                Element connNode = ( Element ) nodeList.item( i );
                String name = "";
                String driver = "";
                String url = "";
                String user = "";
                String pwd = "";
                name    = ( (Element) connNode.getElementsByTagName( "name" ).item(0)).getFirstChild().getNodeValue();
                driver  = ( (Element) connNode.getElementsByTagName( "driver" ).item(0)).getFirstChild().getNodeValue();
                url     = ( (Element) connNode.getElementsByTagName( "url" ).item(0)).getFirstChild().getNodeValue();
                user    = ( (Element) connNode.getElementsByTagName( "user" ).item(0)).getFirstChild().getNodeValue();
                org.w3c.dom.Node node = ( (Element) connNode.getElementsByTagName( "pwd" ).item(0)).getFirstChild();
                
                if( node == null)
                {
                    pwd = "";
                }
                else
                {   
                    pwd = node.getNodeValue();
                    pwd = crypt.decrypt( pwd );
                }    
                // set data
                connections.put( name, new ConnectionInfo( name, driver, url, user, pwd ) );
            }
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        return connections;
    }
    //--------------------------------------------------------------------------
    // Add Connection 
    //--------------------------------------------------------------------------
    public static void addConnection( String name, String driver, String url, String user, String pwd )
    {
        try
        {
            // Hashtable connections = getConnections();
            LinkedHashMap connections = getConnections();
            connections.put( name, new ConnectionInfo( name, driver, url, user, pwd ) );
            saveConnections( connections );
        }
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //--------------------------------------------------------------------------
    // Change Connection 
    //--------------------------------------------------------------------------
    public static void changeConnection( String name, String driver, String url, String user, String pwd )
    {
        try
        {
            LinkedHashMap connections = getConnections();
            connections.remove( name );
            connections.put( name, new ConnectionInfo( name, driver, url, user, pwd ) );
            saveConnections( connections );
        }
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //--------------------------------------------------------------------------
    // Delete Connection 
    //--------------------------------------------------------------------------
    public static void deleteConnection( String name )
    {
        try
        {
            LinkedHashMap connections = getConnections();
            connections.remove( name );
            saveConnections( connections );
        }
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //--------------------------------------------------------------------------
    // Save Connection 
    //--------------------------------------------------------------------------
    private static void saveConnections( LinkedHashMap connections )
    {
        try
        {
            logger.fatal( "Write file: " + configFileName );
            javax.xml.parsers.DocumentBuilderFactory factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
            javax.xml.parsers.DocumentBuilder docBuilder = factory.newDocumentBuilder();
            docBuilder.setErrorHandler( new XmlSaxErrorHandler() ); 
            
            factory.setValidating( true );        // enable Validating with DTD
                       
            org.w3c.dom.Document doc = docBuilder.newDocument();
            Element rootElement     = doc.createElement( "CONNECTIONS" );
            
            doc.appendChild( rootElement );

            Iterator iter = connections.keySet().iterator();
            while( iter.hasNext() )
            {
                ConnectionInfo ci = (ConnectionInfo) connections.get( iter.next() );
            
                Element conElement      = doc.createElement( "connection" );
                Element nameElement     = doc.createElement( "name" );
                Element driverElement   = doc.createElement( "driver" );
                Element urlElement      = doc.createElement( "url" );
                Element userElement     = doc.createElement( "user" );
                Element pwdElement      = doc.createElement( "pwd" );
            
                rootElement.appendChild( conElement );

                conElement.appendChild( nameElement );    
                conElement.appendChild( driverElement );    
                conElement.appendChild( urlElement );    
                conElement.appendChild( userElement );    
                conElement.appendChild( pwdElement );    


                nameElement.appendChild(    doc.createTextNode( ci.getName() ) );
                driverElement.appendChild(  doc.createTextNode( ci.getDriver() ) );
                urlElement.appendChild(     doc.createTextNode( ci.getUrl() ) );
                userElement.appendChild(    doc.createTextNode( ci.getUser()  ) );
                 // we will write all passwords encrypted
                pwdElement.appendChild(     doc.createTextNode( crypt.encrypt( ci.getPwd() ) ) );
            }
            
            
            javax.xml.transform.Transformer t = javax.xml.transform.TransformerFactory.newInstance().newTransformer();
            //
            // Properties see: javax.xml.transform.OutputKeys
            //
            t.setOutputProperty( "method", "xml" );
            t.setOutputProperty( "standalone" , "no" );
            t.setOutputProperty( "indent" , "yes" );
            t.setOutputProperty( "doctype-system" , "diversity_navigator.connections.dtd" );
            // write to file
            t.transform( new javax.xml.transform.dom.DOMSource( doc ),  new javax.xml.transform.stream.StreamResult( new java.io.FileOutputStream( configFileName ) ));
                                    
        }
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }            
    }
    // Errorhandler
    public static class XmlSaxErrorHandler implements org.xml.sax.ErrorHandler
    {        
        
        public void warning( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException 
        {
            // System.out.println( exception.toString() );
            logger.warn( buildMsg( ex ) );
        }
        public void error( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException 
        {
            // System.out.println( exception.toString() );
            logger.error( buildMsg( ex ) );
        }
        public void fatalError( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException
        {
            // System.out.println( exception.toString() );
            logger.fatal( buildMsg( ex ) );            
        }
        // format error message
        private String buildMsg( org.xml.sax.SAXParseException ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            String msg = configFileName + "\n Line: " + ex.getLineNumber() + " Column: " + ex.getColumnNumber() + "\n" + ex.toString();            
            return msg;
        }
    }
}

