package net.metadiversity.diversity.navigator.db;

import javax.swing.event.*;
import javax.swing.tree.*;
import java.util.*;

import java.sql.*;

import net.metadiversity.diversity.navigator.connection.*;

import org.apache.log4j.Logger;

/**
 *
 * @author  didi
 */
public class DBNode 
{
    static Logger logger = Logger.getLogger( DBNode.class ); 
    
    public static final int NODE_TYPE_CAPTION_CONNECTIONS    = 1;
    public static final int NODE_TYPE_DATABASE               = 2;
    public static final int NODE_TYPE_TABLE                  = 3;
    public static final int NODE_TYPE_FIELD                  = 4;

    String  name = "";
    int     nodeType = 0;
    Object  data = null;
    Object  data2 = null;
    Vector  childs = null;
    DBNode  parentNode = null;
    ConnectionsPane inPane;

    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public DBNode( ConnectionsPane inPane, DBNode parentNode, int nodeType, String name ) 
    {
	this( inPane, parentNode, nodeType, name, null );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public DBNode( ConnectionsPane inPane, DBNode parentNode, int nodeType, String name, Object data ) 
    {
	this.nodeType   = nodeType;
	this.name       = name;
	this.parentNode = parentNode;
	this.inPane     = inPane;
	this.data       = data;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public String getName()
    {
        return this.name;
    }
    public int getNodeType()
    {
        return this.nodeType;
    }
    public Object getData()
    {
        return data;
    }
    public void setData(Object data)
    {
        this.data = data;
    }
    public Object getData2()
    {
        return data2;
    }
    public void setData2(Object data)
    {
        this.data2 = data;
    }
    public DBNode getParentNode()
    {
        return this.parentNode;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public ConnectionInfo getConnectionInfo() 
    {
        if( nodeType == NODE_TYPE_CAPTION_CONNECTIONS ) 
        {
            return null;
        }
        DBNode node = this;
        while( node.getNodeType() != NODE_TYPE_DATABASE ) 
        {
            node = node.getParentNode();
        }
        return ( ConnectionInfo ) node.getData();
	}
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public Connection getConnection() 
    {
        if( nodeType == NODE_TYPE_CAPTION_CONNECTIONS ) 
        {
            return null;
        }
        DBNode node = this;
        while( node.getNodeType() != NODE_TYPE_DATABASE )
        {   
            node = node.getParentNode();
        }
        if( node.getData2() == null )
        {
            node.load();
        }
        return ( Connection ) node.getData2();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public String getTable() 
    {
        if( nodeType == NODE_TYPE_CAPTION_CONNECTIONS || nodeType == NODE_TYPE_DATABASE ) 
        {
            return null;
        }
        DBNode node = this;
        while( node.getNodeType() != NODE_TYPE_TABLE )
        {
            node = node.getParentNode();
        }
        return node.getName();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private void load() 
    {
	try 
        {
            if( childs != null) 
            {
                 return;
            }
            Connection Con = null;
            switch( this.getNodeType() ) 
            {
		case NODE_TYPE_CAPTION_CONNECTIONS:
                    childs = new Vector();
                    Map connections = ConnectionsManager.getConnections();
                    Iterator iter = connections.keySet().iterator();
                    while( iter.hasNext() )
                    {
                        ConnectionInfo ci = (ConnectionInfo) connections.get( iter.next() );                        
                        childs.add( new DBNode( inPane, this, NODE_TYPE_DATABASE, ci.getName(), ci));
                    }
                    break;
                case NODE_TYPE_DATABASE:
                    childs = new Vector();
                    ConnectionInfo ci = getConnectionInfo();
                    /* ???
                    try 
                    {
                        inPane.getMainWindow().setConnectionStatus("Loading driver ...");
                        Class.forName( ci.getDriver() );
                    } 
                    catch( Exception ex ) 
                    {
                        inPane.getMainWindow().setConnectionError( "Can't load driver: " + ci.getDriver() );
                        // ex.printStackTrace();
                        return;
                    }
                     */
                    try 
                    {
                        inPane.getMainWindow().setConnectionStatus( "Connecting ..." );
                        Con = DriverManager.getConnection( ci.getUrl(), ci.getUser(), ci.getPwd() );
                        setData2(Con);
                    } 
                    catch( Exception ex ) 
                    {
                        inPane.getMainWindow().setConnectionError( "Can't connect to: " + ci.getUrl() );
			// ex.printStackTrace();
			return;
                    }					
                    try 
                    {
                        inPane.getMainWindow().setConnectionStatus( "Retrieving tables ..." );
                        DatabaseMetaData MD = Con.getMetaData();
                        String[] Types = { "TABLE" };
                        ResultSet Tables = MD.getTables(null, null, null, Types);
                        while( Tables.next() ) 
                        {
                            String TableName = Tables.getString( "TABLE_NAME" );
                            DBNode child = new DBNode(inPane, this, NODE_TYPE_TABLE, TableName);
                            childs.add(child);
                        }
                        Tables.close();
                    } 
                    catch( Exception ex ) 
                    {
                        inPane.getMainWindow().setConnectionError( "Can't get tables: " + ex.getMessage());
                        ex.printStackTrace();
                        return;
                    }
                    inPane.getMainWindow().setConnectionStatus( "Connected to " + ci.getName() );
                    break;

                case NODE_TYPE_TABLE:

                    childs = new Vector();
                    Con = (Connection)getParentNode().getData2();
                    try 
                    {
                        inPane.getMainWindow().setConnectionStatus( "Retrieving fields ..." );
                        DatabaseMetaData MD = Con.getMetaData();
                        String TableName = getName();
                        ResultSet Columns = MD.getColumns( null, null, TableName, null );
                        while( Columns.next() ) 
                        {
                            String ColumnName = Columns.getString( "COLUMN_NAME" );							
                            DBNode child = new DBNode( inPane, this, NODE_TYPE_FIELD, ColumnName );
                            childs.add( child );
                        }
                        Columns.close();
                    } 
                    catch( Exception ex ) 
                    {
                        logger.error( ex.getMessage(), ex.fillInStackTrace() );
                        inPane.getMainWindow().setConnectionStatus( "Can't get fields: " + ex.getMessage() );
                        ex.printStackTrace();
                        return;
                    }
                    inPane.getMainWindow().setConnectionStatus( "Connected to " + getConnectionInfo().getName() );
                    break;
            }
        } 
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //----------------------------------------------------------------------
    // Returns true if the receiver is a leaf.
    //----------------------------------------------------------------------
    public boolean isLeaf() 
    {
        return getNodeType() == NODE_TYPE_FIELD;
    }
    //----------------------------------------------------------------------
    // Returns the number of children TreeNodes the receiver contains.
    //----------------------------------------------------------------------
    public int getChildCount() 
    {
        load();
        return childs.size();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public DBNode getChild(int i) 
    {
        load();
        return (DBNode)childs.elementAt(i);
    }
    //----------------------------------------------------------------------
    // Returns the child at the specified index in this node's child array.
    //----------------------------------------------------------------------
    public int getChildIndex(Object ochild) 
    {
        load();
        DBNode child = (DBNode)ochild;
        for( int i=0; i < getChildCount(); i++ ) 
        {
            if( getChild(i).getName().equals(child.getName()) ) 
            {    
                return i;
            }
        }
        return -1;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public String toString() 
    {
        return this.getName();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void reloadChilds() 
    {
        childs = null;
    }
}

