package net.metadiversity.diversity.navigator.table;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.sql.*;
import javax.swing.table.*;
import javax.swing.JTable;
import java.util.*;

// XML

// import javax.xml.parsers.*;
// import org.xml.sax.*;
// import org.xml.sax.helpers.*;
// import org.w3c.dom.*;

// JDOM

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Comment;
import org.jdom.output.XMLOutputter;
import org.jdom.input.SAXBuilder;

import net.metadiversity.diversity.navigator.*;
import net.metadiversity.diversity.navigator.connection.*;

import org.apache.log4j.Logger;

public class TableFrame extends JInternalFrame 
{
    static Logger logger = Logger.getLogger( TableFrame.class ); 

    private ConnectionInfo      ci;
    private Connection          con;
    private String              tableName;
    private JTabbedPane         frames;
    private TableTableModel     resultsModel;
    private JTable              results;
    private DiversityNavigator  mainWindow;
    private JTextArea           outputTextArea;

    final private static String XML_ENCODING = "UTF-8";

    public TableFrame( ConnectionInfo ci, Connection con, String tableName, DiversityNavigator mainWindow ) 
    {
        super( tableName, true, true, true, true );

        this.ci = ci;
        this.con = con;
        this.mainWindow = mainWindow;
        this.tableName = tableName;
        getContentPane().setLayout( new BorderLayout() );
        String Path = "";

        try 
        {
            Path = new java.io.File( Class.forName( "net.metadiversity.diversity.navigator.connection.ConnectionsManager" ).getProtectionDomain().getCodeSource().getLocation().getFile() ).getCanonicalPath();
        } 
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        //---------------------
        // Set Diversity Icon
        //----------------------
        this.setFrameIcon( DiversityNavigatorProperties.getDiversityIcon()  );

        //--------------------------------------------------------------
        // Frames
        //--------------------------------------------------------------
        frames = new JTabbedPane();
        getContentPane().add( frames, BorderLayout.CENTER );

        //--------------------------------------------------------------
        // Results Pane
        //--------------------------------------------------------------
        JPanel resultsPane      = new JPanel();
        resultsModel            = new TableTableModel( con, tableName, mainWindow );

        // Sorter for table
        final TableSorter sorter      = new TableSorter( resultsModel ); //ADDED THIS
        results                 = new JTable( sorter );
        sorter.addMouseListenerToHeaderInTable( results ); //ADDED THIS
        JScrollPane rsp         = new JScrollPane( results );

        resultsPane.setLayout( new BorderLayout());
        results.setAutoResizeMode( JTable.AUTO_RESIZE_OFF ); 
        resultsModel.setInTable( results );
        resultsPane.add( rsp, BorderLayout.CENTER );
        frames.add( resultsPane, "Table Data" );

        //--------------------------------------------------------------
        // Details Panel
        //--------------------------------------------------------------
/* 
 * old text version
 *       
        JPanel detailsPanel         = new JPanel();
        JTextArea detailsTextArea   = new JTextArea( resultsModel.getTableDetails() );
        JScrollPane dsp             = new JScrollPane( detailsTextArea );
        detailsPanel.setLayout( new BorderLayout());
        detailsTextArea.setEditable( false );
        detailsPanel.add( dsp, BorderLayout.CENTER );
*/
        JPanel detailsPanel         = new DetailsPanel( resultsModel );
        frames.add( detailsPanel, "Details" );

        //--------------------------------------------------------------
        // Filter Panel
        //--------------------------------------------------------------
        JPanel filterPanel         = new FilterPanel( resultsModel );
        frames.add( filterPanel, "Filter" );
        //--------------------------------------------------------------
        // Edit Pane
        //--------------------------------------------------------------
/*
        JPanel editPanel = new JPanel();
        editPanel.setLayout( new BorderLayout() );
        JTextArea editTextArea = new JTextArea( ResultsModel.getTableDetails() );
        editTextArea.setEditable( false );
        JScrollPane editSP = new JScrollPane( editTextArea );
        .add( editSP, BorderLayout.CENTER );
        Frames.add( editPanel, "Edit" );
*/
        //--------------------------------------------------------------
        // Output Pane
        //--------------------------------------------------------------
        JPanel outputPanel      = new JPanel();
        outputTextArea          = new JTextArea();
        JScrollPane osp         = new JScrollPane( outputTextArea );

        outputPanel.setLayout( new BorderLayout());
        outputTextArea.setEditable( false );
        outputPanel.add( osp, BorderLayout.CENTER);
        frames.add( outputPanel, "Output to server" );

        //--------------------------------------------------------------
        // Buttons bar
        //--------------------------------------------------------------
        JToolBar Buttons = new JToolBar( "Tables" );
        JButton cmdSaveChanges = new JButton(DiversityNavigatorProperties.getSaveIcon() );
        cmdSaveChanges.setToolTipText( "Apply changes to database" );
        cmdSaveChanges.addActionListener( new ActionListener() 
        {
                public void actionPerformed(ActionEvent e) 
                {
                        save();
                }
        });
        Buttons.add( cmdSaveChanges) ;
        Buttons.add( Box.createHorizontalStrut(10)) ;
        // New
        JButton cmdNewRow = new JButton( DiversityNavigatorProperties.getNewIcon() );
        cmdNewRow.setToolTipText( "Append a new row at the end of the table" );
        cmdNewRow.addActionListener( new ActionListener() 
        {
                public void actionPerformed( ActionEvent e ) 
                {
                        newRow();
                }
        });
        Buttons.add( cmdNewRow );
        // Delete
        JButton cmdDeleteRow = new JButton( DiversityNavigatorProperties.getDeleteIcon() );
        cmdDeleteRow.setToolTipText( "Deletes current row from table" );
        cmdDeleteRow.addActionListener( new ActionListener() 
        {
                public void actionPerformed( ActionEvent e ) 
                {
                        deleteRow( sorter.getDeleteRowIndex( results.getSelectedRow() ));
                }
        });
        Buttons.add( cmdDeleteRow );

        getContentPane().add(Buttons, BorderLayout.NORTH);
        setBounds( 0, 0, 400, 300 );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public DiversityNavigator getMainWindow() 
    {
        return mainWindow;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void save() 
    {
        frames.setSelectedIndex( 3 );
        resultsModel.save( con, outputTextArea );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private void newRow() 
    {
        resultsModel.newRow();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private void deleteRow( int row ) 
    {
//        int row = results.getSelectedRow();
// didi        sorter.
        if( row >= 0 )
        {
           resultsModel.deleteRow( row );
        }
    }
    //----------------------------------------------------------------------
    // Export XML with JDOM
    //----------------------------------------------------------------------
    public void exportXmlTable( java.io.File file ) 
    {
        // Create a new JDOM Document             
        Element     root = new Element( "tabledata" );
        Document    doc  = new Document( root );

        java.util.Date now = new java.util.Date();
/* deprecated    
        String  str = "Created from Diversity Navigator on " + now.t.toGMTString();
 */
        java.text.DateFormat df = java.text.DateFormat.getDateTimeInstance();
        String  str = "Created from Diversity Navigator on " + df.format( now );
        
        Comment     comment = new Comment( str );
        root.addContent( comment );
        for( int i=0; i < resultsModel.getRowCount(); i++ ) 
        {
            Element element = new Element( "row" );

            for( int c = 0; c < resultsModel.getColumnCount(); c++ ) 
            {
                Element subElement = new Element( resultsModel.getColumnName( c ) );                            
                Object  obj = resultsModel.getValueAt( i, c );
                String data;
                if( obj == null )
                {
                    data = "NULL";
                }
                else
                {
                    data = obj.toString();
                }
                subElement.setAttribute( "data", data );
                element.addContent( subElement );
            }
            root.addContent( element );
        }

        try 
        {
            XMLOutputter out = new XMLOutputter( "  ", true ); 
            java.io.FileOutputStream output = new java.io.FileOutputStream( file );

            out.output( doc, output );
            output.close();
        } 
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            // mainWindow.setError( ex.getMessage() );                        
        }
    }
    //----------------------------------------------------------------------
    // Export as TAB seperated list
    //----------------------------------------------------------------------
    public void exportTabTable( java.io.File file ) 
    {            
        String delimiter = "\t";                       
        StringBuffer buffer = new StringBuffer();
        for( int i = 0; i < resultsModel.getRowCount(); i++ ) 
        {
            // for each col build one output line
            for( int col = 0; col < resultsModel.getColumnCount(); col++ ) 
            {
                if( col != 0 )
                {
                    buffer.append( delimiter );
                }
                buffer.append( resultsModel.getValueAt( i, col ));
            }
            buffer.append( "\n" );
        }
        try 
        {
            java.io.FileOutputStream   fos = new java.io.FileOutputStream( file );
            java.io.OutputStreamWriter osw = new java.io.OutputStreamWriter( fos, "UTF8" ); 
            java.io.BufferedWriter output  = new java.io.BufferedWriter( osw );
            output.write( buffer.toString() );
            output.close();
        }
        catch( java.io.IOException ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            // mainWindow.setError( ex.getMessage() );                        
        }
    }
    //----------------------------------------------------------------------
    // Import XML with JDOM
    //----------------------------------------------------------------------
    public void importXmlTable(java.io.File file ) 
    {
        try 
        {
            // Load XML into JDOM Document                
            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build( file );

            //  List of rows
            java.util.List rows = doc.getRootElement().getChildren();
            for( Iterator i = rows.iterator(); i.hasNext(); )
            {
                Element rowNode = (Element) i.next();                
                String  row = rowNode.getName();

                if( row.compareTo( "row" ) == 0 )
                {
                    Vector rowData = new Vector();
                    for( int c = 0; c < resultsModel.getColumnCount(); c++ ) 
                    {
                        String value = ((Element) rowNode.getChild( resultsModel.getColumnName( c ) )).getAttributeValue( "data" );
                        rowData.add( value );
                        // System.out.println( resultsModel.getColumnName( c ) + " = " + value );
                    }
                    resultsModel.addRow( rowData );

                }
            }
        } 
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            // mainWindow.setError( ex.getMessage() );                        
        }
    }
    //----------------------------------------------------------------------
    // Import as TAB seperated list
    //----------------------------------------------------------------------
/*        
    public void importTable(java.io.File file ) 
    {
        try 
        {
            javax.xml.parsers.DocumentBuilder DocBuilder = javax.xml.parsers.DocumentBuilderFactory.newInstance().newDocumentBuilder();
            org.w3c.dom.Document Doc = DocBuilder.parse( file );
            Element root = Doc.getDocumentElement();
            NodeList RowList = root.getElementsByTagName( "ROW" );
            for(int i=0; i < RowList.getLength(); i++ ) 
            {
                Element RowNode = (Element) RowList.item(i);
                Vector Row = new Vector();
                for (int c=0; c < resultsModel.getColumnCount(); c++) 
                {
                        String value = ((Element) RowNode.getElementsByTagName( resultsModel.getColumnName(c)).item(0)).getFirstChild().getNodeValue();
                        Row.add(value);
                }
                resultsModel.addRow( Row );
            }
        } 
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            mainWindow.setError( ex.getMessage() );                        
        }
    }
*/
    //----------------------------------------------------------------------
    // Import from file
    //----------------------------------------------------------------------
    public void importTabTable( java.io.File file ) 
    {
        String delimiter = "\t";           
        try 
        {
            java.io.BufferedReader  input     = null;                
            //---------------------
            // read data into table
            //---------------------
            try
            {
                input = new java.io.BufferedReader( new java.io.InputStreamReader( new java.io.FileInputStream( file ), "UTF-8" ) );
            }
            catch( java.io.UnsupportedEncodingException ex )
            {
                logger.error( ex.getMessage(), ex.fillInStackTrace() );
                JOptionPane.showMessageDialog( null, ex.toString(), "Error" , JOptionPane.ERROR_MESSAGE );
                return;
            }
            int fieldCount = resultsModel.getColumnCount();                    
            Vector types = new Vector();                    
            for( int i = 0; i < fieldCount; i++ )
            {
                 types.add( i, new Short( resultsModel.getColumnClassType( i ) ) );
            }
            //-----------
            // read lines
            //-----------                                
            while( input.ready() )
            {
                String line = input.readLine();
                if( line == null )
                {
                    input.close();
                    return;
                }
                logger.debug( line );
                Vector row = new Vector();                        
                // parse all fields in input line.                        
                java.util.StringTokenizer tokens = new java.util.StringTokenizer( line, delimiter, true );
                String value = "";
                int readCount = 0;
                // get data for all fields
                while( tokens.hasMoreTokens() ) 
                {
                    readCount++;
                    if( readCount > fieldCount )
                    {
                        // mainWindow.setError( ex.getMessage() );                        
                        break;
                    }                        
                    // get Data Type for this collumn   
                    int type = ((Short) types.get( readCount - 1 )).intValue();

                    value = tokens.nextToken();
                    // logger.debug( "token: " + value );

                    if( value.compareTo( delimiter ) == 0 ) 
                    {
                        // if there is no Data for this collumn
                        switch( type )
                        {
                        case java.sql.Types.BIT :        
                                // logger.debug( "Type is BIT" );
                                row.add( new net.metadiversity.diversity.navigator.types.DnBoolean( 0 ) );
                                break;
                        case java.sql.Types.INTEGER :        
                        case java.sql.Types.TINYINT :        
                                // logger.debug( "Type is INT" );
                                row.add( new Integer( 0 ) );
                                break;
                        case java.sql.Types.DATE :
                        case java.sql.Types.TIME :
                        case java.sql.Types.TIMESTAMP :
                                row.add( "" );
                                break;                                    
                        default :
                                // logger.debug( "Type is STRING" );
                                row.add( "" );                                        
                                break;
                        }    
                    }
                    else
                    {
                        switch( type )
                        {
                        case java.sql.Types.BIT :                                        
                            int accessVal = java.lang.Integer.parseInt( value );
                            switch( accessVal )
                            {
                                case 0:
                                    row.add( new net.metadiversity.diversity.navigator.types.DnBoolean( -1 ) );
                                    break;
                                case 1:
                                    row.add( new net.metadiversity.diversity.navigator.types.DnBoolean(  1 ) );
                                    break;
                                default:
                                    row.add( new net.metadiversity.diversity.navigator.types.DnBoolean(  0 ) );
                                    break;
                            }
                            break;
                        case java.sql.Types.INTEGER :        
                        case java.sql.Types.TINYINT : 
                            row.add( new Integer( java.lang.Integer.parseInt( value ) ) );
                            break;
                        /*            
                        case java.sql.Types.DATE :
                        case java.sql.Types.TIME :
                        case java.sql.Types.TIMESTAMP :
                         */
                        default :
                            if( value.compareToIgnoreCase( "null" ) == 0 )
                            {
                                row.add( "" );
                            }    
                            else
                            {    
                                row.add( value );
                            }    
                            break;
                        }    
                        // logger.debug( value );
                        // read next delimmiter
                        if( tokens.hasMoreTokens() )
                        {
                            value = tokens.nextToken();
                            if( ! value.equalsIgnoreCase( "\t" ) )
                            {    
                                logger.debug( "invalid delimmiter token: " + value );
                            }    
                        }
                    }
                }                            
                for(int i = readCount; i < fieldCount; i++ )
                {
                    row.add( "null" );
                }
                resultsModel.addRow( row );
            }
        } 
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            // mainWindow.setError( ex.getMessage() );                        
        }
    }
}
